@@ -1,11 +1,11 @@
 package Chemistry::Ring::Find;
 
-$VERSION = 0.10;
-# $Id: Find.pm,v 1.1.1.1 2004/06/16 19:38:14 ivan Exp $
+$VERSION = 0.16;
+# $Id: Find.pm,v 1.4 2004/08/03 01:09:32 ivan Exp $
 
 =head1 NAME
 
-Chemistry::Ring::Find
+Chemistry::Ring::Find - Find the rings (cycles) in a molecule
 
 =head1 SYNOPSIS
 
@@ -19,12 +19,32 @@ Chemistry::Ring::Find
 
     # see below for more options
 
+    # find the six 4-atom rings in cubane
+    @rings = find_rings($cubane);
+
+    # find a cubane SSSR with five rings
+    @rings = find_rings($cubane, sssr => 1);
+
 =head1 DESCRIPTION
 
-This module implements a simple breadth-first ring finding algorithm. It
-does not find all the rings in the structure; it only finds the rings that 
-include a starting atom or bond. Future versions may find all the rings in 
-the molecule, or perhaps the Smallest Set of Smallest Rings.
+The Chemistry::Ring::Find module implements a breadth-first ring finding
+algorithm, and it can find all rings that contain a given atom or bond, the
+Smallest Set of Smallest Rings (SSSR), or the "almost SSSR", which is an
+unambiguous set of rings for cases such as cubane.The algorithms are  based on
+ideas from:
+
+1) Leach, A. R.; Dolata, D. P.; Prout, P. Automated Conformational Analysis and
+Structure Generation: Algorithms for Molecular Perception J. Chem. Inf. Comput.
+Sci. 1990, 30, 316-324
+
+2) Figueras, J. Ring perception using breadth-first search. J. Chem. Inf.
+Comput.  Sci. 1996, 36, 986-991.
+
+Ref. 2 is only used for find_ring, not for find_rings, because it has been
+shown that the overall SSSR method in ref 2 has bugs. Ref 1 inspired
+find_rings, which depends on find_ring.
+
+This module is part of the PerlMol project, L<http://www.perlmol.org/>.
 
 =head1 FUNCTIONS
 
@@ -41,7 +61,7 @@ use warnings;
 use Chemistry::Ring;
 
 our @ISA = qw(Exporter);
-our @EXPORT_OK = qw(find_ring);
+our @EXPORT_OK = qw(find_ring find_rings);
 our %EXPORT_TAGS = ( all => \@EXPORT_OK ); 
 
 our $DEBUG = 0;
@@ -56,7 +76,8 @@ Returns a Chemistry::Ring object. Options:
 =item all
 
 If true, find all the rings containing $origin. If false, return the first ring
-found. Defaults to false.
+found. Defaults to false. "All" is supposed to include only "simple" rings,
+that is, rings that are not a combination of smaller rings. 
 
 =item min
 
@@ -185,33 +206,109 @@ sub print_path {
     $ret .= "\n";
 }
 
+# contains_ring($atoms, $rings)
+# returns true if one of the rings in the array ref $rings is a proper subset
+# of the atom list in the array ref $atom.
 sub contains_ring {
     my ($atoms, $rings) = @_;
     my %seen;
     @seen{@$atoms} = ();
     for my $ring (@$rings) {
-        my $ok = 0;
+        my $unique_atoms = $ring->atoms; 
+        next if $unique_atoms >= @$atoms; # ring is same size or bigger
         # make sure that $ring has at least one atom not in $atoms
         for my $atom ($ring->atoms) {
-            $ok = 1, last unless exists $seen{$atom}; 
+            if (exists $seen{$atom}) {
+                $unique_atoms--;
+            } else {
+                last; # there's at least one unique atom!
+            }
         }
-        return 1 unless $ok;
+        return 1 unless $unique_atoms;
     }
     0;
 }
 
+=item @rings = find_rings($mol, %options)
+
+Find "all" the rings in the molecule. In general it return the Smallest Set of
+Smallest Rings (SSSR). However, since it is well known that the SSSR is not
+unique for molecules such as cubane (where the SSSR consists of five
+unspecified four-member rings, even if the symmetry of the molecule would
+suggest that the six faces of the cube are equivalent), in such cases
+find_rings will return a non-ambiguous "non-smallest" set of smallest rings,
+unless the "sssr" option is given. For example,
+
+    @rings = find_rings($cubane);
+    # returns SIX four-member rings
+
+    @rings = find_rings($cubane, sssr => 1);
+    # returns FIVE four-member rings (an unspecified subset of
+    # the six rings above.)
+
+=cut
+
+sub find_rings {
+    my ($mol, %opts) = @_;
+    my @ring_bonds = find_ring_bonds($mol, \%opts, $mol->atoms(1), {});
+    my @rings;
+    my $n_rings = @ring_bonds;
+    #print "cyclomatic number=$n_rings\n";
+    for my $ring_bond (@ring_bonds) {
+        push @rings, find_ring($ring_bond, all => 1)
+    }
+    my %seen;
+    my @ring_keys = map { join " ", sort $_->atoms } @rings;
+    @seen{@ring_keys} = @rings;
+    @rings = sort { $a->atoms <=> $b->atoms } values %seen;
+    if (!$opts{sssr} and @rings > $n_rings) {
+        $n_rings++ if $rings[$n_rings]->atoms == $rings[$n_rings-1]->atoms;
+    }
+    splice @rings, $n_rings;
+    @rings;
+}
+
+sub find_ring_bonds {
+    my ($mol, $opts, $atom, $visited) = @_;
+
+    my @ring_bonds;
+    $visited->{$atom}  = 1;
+    for my $bn ($atom->bonds_neighbors) {
+        my $nei  = $bn->{to};
+        my $bond = $bn->{bond};
+        next if $visited->{$bond};
+        $visited->{$bond}  = 1;
+        if ($visited->{$nei}) { # closed ring
+            #print "closing ring\n";
+            push @ring_bonds, $bond;
+        } else {
+            push @ring_bonds, 
+                find_ring_bonds($mol, $opts, $nei, $visited);
+        }
+    }
+    @ring_bonds;
+}
+
 1;
 
 =back
 
+=head1 BUGS
+
+The "all" option in find_ring doesn't quite work as expected. It finds all
+simple rings and some bridged rings. It never finds fused rings (which is good).
+
+=head1 VERSION
+
+0.16
+
 =head1 SEE ALSO
 
-L<Chemistry::Ring>, L<Chemistry::Mol>, L<Chemistry::Atom>, L<Chemistry::Bond>
-L<Math::VectorReal>.
+L<Chemistry::Ring>, L<http://www.perlmol.org>.
 
 =head1 AUTHOR
 
-Ivan Tubert E<lt>itub@cpan.orgE<gt>
+Ivan Tubert-Brohman E<lt>itub@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
